// Copyright 2011 Julian Phillips.  All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package py

// #include "utils.h"
//import "C"

import (
	"fmt"
	"unsafe"
)

type Int struct {
	AbstractObject
	NumberProtocol
	o _PyIntObject
}

// IntType is the Type object that represents the Int type.
var IntType = (*Type)(unsafe.Pointer(&*_PyInt_Type))

func intCheck(obj Object) bool {
	if obj == nil {
		return false
	}
	return _intCheck(c(obj)) != 0
}

func newInt(obj *_PyObject) *Int {
	return (*Int)(unsafe.Pointer(obj))
}

func NewInt(i int) *Int {
	return newInt(_PyInt_FromLong(int64(i)))
}

func NewInt64(i int64) *Int {
	return newInt(_PyInt_FromSsize_t(int64(i)))
}

func (i *Int) Int() int {
	return int(_PyInt_AsLong(c(i)))
}

func (i *Int) String() string {
	if i == nil {
		return "<nil>"
	}
	return fmt.Sprintf("%v", i.Int())
}
